﻿-- ------------------------------------------------------------------------------------- --
-- 					Scroll Master - AddOn by Sapu (sapu94@gmail.com)			 		 --
--             http://wow.curse.com/downloads/wow-addons/details/slippy.aspx             --
-- ------------------------------------------------------------------------------------- --

-- The following functions are contained attached to this file:
-- Enchanting:OpenCrafting() - opens the enchanting window and removes any filters on the shown crafts so that everything is visible
-- Enchanting:IsEnchanter() - determines if the player is an enchanter

-- The following "global" (within the addon) variables are initialized in this file:
-- Enchanting.vellumID - itemID for Enchanting Vellum

-- ===================================================================================== --


-- load the parent file (ScrollMaster) into a local variable and register this file as a module
local TSM = select(2, ...)
local Enchanting = TSM:NewModule("Enchanting", "AceEvent-3.0")

local aceL = LibStub("AceLocale-3.0"):GetLocale("TradeSkillMaster_Crafting") -- loads the localization table
local debug = function(...) TSM:Debug(...) end -- for debugging

local function L(phrase)
	--TSM.lTable[phrase] = true
	return aceL[phrase]
end

-- opens the enchanting window and removes any filters on the shown crafts so that everything is visible
function Enchanting:OpenCrafting(nextFunc)
	CloseTradeSkill()
	local spellName
	if TSM.mode == "Enchanting" then
		spellName = GetSpellInfo(51313) --Inscription = GetSpellInfo(45363)
	end
	Enchanting:RegisterEvent("TRADE_SKILL_SHOW", function()
			Enchanting:UnregisterEvent("TRADE_SKILL_SHOW")
			if nextFunc then nextFunc() end
		end)
	CastSpellByName(spellName) -- opens enchanting
end

-- determines if the player is an enchanter (either /450 or /525)
function Enchanting:IsEnchanter()
	local enchantIDs = {7411, 7412, 7413, 13920, 28029, 51313, 74258}
	for _, id in pairs(enchantIDs) do
		if IsSpellKnown(id) then return true end
	end
end

function Enchanting:GetSlot(itemID)
	if not itemID then return end
	return Enchanting.slot[itemID]
end

Enchanting.vellumID = 38682

Enchanting.slotList = {L("2H Weapon"), L("Boots"), L("Bracers"), L("Chest"), L("Cloak"), L("Gloves"), L("Shield"), L("Staff"), L("Weapon")}

-- looks up the itemID of the scroll that the enchant makes
-- index = spellID of the enchant
-- value = itemID of scroll
Enchanting.itemID = {
	[44488] = 38953,
	[7426] = 38767,
	[42974] = 38948,
	[7428] = 38768,
	[44528] = 38966,
	[60609] = 44456,
	[13815] = 38827,
	[13817] = 38828,
	[59621] = 44493,
	[44584] = 38974,
	[44592] = 38979,
	[27899] = 38897,
	[22750] = 38878,
	[44616] = 38987,
	[20031] = 38870,
	[7443] = 38769,
	[46578] = 38998,
	[20010] = 38854,
	[13841] = 38831,
	[25073] = 38886,
	[23802] = 38882,
	[25081] = 38891,
	[13898] = 38838,
	[44513] = 38964,
	[33994] = 38932,
	[34002] = 38938,
	[7454] = 38770,
	[13695] = 38822,
	[27914] = 38902,
	[20013] = 38857,
	[20017] = 38861,
	[13612] = 38800,
	[20025] = 38865,
	[20029] = 38868,
	[20033] = 38872,
	[13620] = 38802,
	[27946] = 38906,
	[27950] = 38909,
	[27954] = 38910,
	[27958] = 38912,
	[27962] = 38915,
	[13887] = 38836,
	[44593] = 38980,
	[13655] = 38814,
	[13635] = 38806,
	[13640] = 38808,
	[44625] = 38990,
	[13644] = 38810,
	[13646] = 38811,
	[13648] = 38812,
	[13905] = 38839,
	[13538] = 38798,
	[13421] = 38790,
	[7857] = 38782,
	[64579] = 46098,
	[13915] = 38840,
	[13917] = 38841,
	[34003] = 38939,
	[7766] = 38774,
	[7420] = 38766,
	[27968] = 38918,
	[47900] = 39005,
	[7745] = 38772,
	[44506] = 38960,
	[13933] = 38843,
	[13935] = 38844,
	[13937] = 38845,
	[13939] = 38846,
	[13941] = 38847,
	[13943] = 38848,
	[13945] = 38849,
	[13947] = 38850,
	[25074] = 38887,
	[20014] = 38858,
	[13698] = 38823,
	[13700] = 38824,
	[44595] = 38981,
	[44582] = 38973,
	[60691] = 44463,
	[13536] = 38797,
	[60707] = 44466,
	[44575] = 44815,
	[44555] = 38968,
	[23799] = 38879,
	[25078] = 38888,
	[25082] = 38892,
	[25086] = 38895,
	[60763] = 44469,
	[33996] = 38934,
	[34004] = 38940,
	[60616] = 38971,
	[27911] = 38900,
	[44483] = 38950,
	[7776] = 38776,
	[33995] = 38933,
	[13485] = 38792,
	[7779] = 38777,
	[20030] = 38869,
	[13746] = 38825,
	[7782] = 38778,
	[27947] = 38907,
	[27951] = 37603,
	[27984] = 38925,
	[13501] = 38793,
	[13503] = 38794,
	[7788] = 38780,
	[27971] = 38919,
	[60668] = 44458,
	[27967] = 38917,
	[27913] = 38901,
	[60692] = 44465,
	[44635] = 38997,
	[44489] = 38954,
	[7786] = 38779,
	[28003] = 38926,
	[20009] = 38853,
	[13890] = 38837,
	[13529] = 38796,
	[13822] = 38829,
	[13693] = 38821,
	[33997] = 38935,
	[34005] = 38941,
	[13794] = 38826,
	[13626] = 38804,
	[44484] = 38951,
	[44492] = 38955,
	[44500] = 38959,
	[44508] = 38961,
	[71692] = 50816,
	[44524] = 38965,
	[13419] = 38789,
	[7793] = 38781,
	[7748] = 38773,
	[44556] = 38969,
	[60653] = 44455,
	[47899] = 39004,
	[59625] = 43987,
	[44588] = 38975,
	[44596] = 38982,
	[63746] = 45628,
	[47672] = 39001,
	[23803] = 38883,
	[44576] = 38972,
	[13836] = 38830,
	[44598] = 38984,
	[27982] = 38924,
	[42620] = 38947,
	[23800] = 38880,
	[13846] = 38832,
	[25083] = 38893,
	[13642] = 38809,
	[33990] = 38928,
	[23804] = 38884,
	[34006] = 38942,
	[13858] = 38833,
	[60714] = 44467,
	[20011] = 38855,
	[20015] = 38859,
	[47051] = 39000,
	[20023] = 38863,
	[60767] = 44470,
	[13617] = 38801,
	[20035] = 38874,
	[27944] = 38904,
	[27948] = 38908,
	[13689] = 38820,
	[13882] = 38835,
	[27960] = 38913,
	[13631] = 38805,
	[44589] = 38976,
	[27972] = 38920,
	[13637] = 38807,
	[13380] = 38788,
	[44621] = 38988,
	[44629] = 38991,
	[13663] = 38818,
	[7859] = 38783,
	[44633] = 38995,
	[7861] = 38784,
	[13653] = 38813,
	[7863] = 38785,
	[13657] = 38815,
	[13659] = 38816,
	[13661] = 38817,
	[7867] = 38786,
	[13948] = 38851,
	[20034] = 38873,
	[20012] = 38856,
	[27906] = 38899,
	[44494] = 38956,
	[46594] = 38999,
	[44510] = 38963,
	[60621] = 44453,
	[62256] = 44947,
	[7771] = 38775,
	[13622] = 38803,
	[13687] = 38819,
	[60623] = 38986,
	[13378] = 38787,
	[59619] = 44497,
	[27837] = 38896,
	[44590] = 38977,
	[60663] = 44457,
	[22749] = 38877,
	[7457] = 38771,
	[47766] = 39002,
	[44630] = 38992,
	[44383] = 38949,
	[13931] = 38842,
	[44509] = 38962,
	[25072] = 38885,
	[23801] = 38881,
	[13464] = 38791,
	[25084] = 38894,
	[34010] = 38946,
	[33992] = 38930,
	[25080] = 38890,
	[27905] = 38898,
	[27975] = 38921,
	[20008] = 38852,
	[27917] = 38903,
	[20016] = 38860,
	[20020] = 38862,
	[20024] = 38864,
	[20028] = 38867,
	[20032] = 38871,
	[20036] = 38875,
	[27945] = 38905,
	[64441] = 46026,
	[13868] = 38834,
	[27957] = 38911,
	[27961] = 38914,
	[47898] = 39003,
	[44591] = 38978,
	[62959] = 45060,
	[27977] = 38922,
	[27981] = 38923,
	[44623] = 38989,
	[44631] = 38993,
	[60606] = 44449,
	[20026] = 38866,
	[13522] = 38795,
	[33991] = 38929,
	[33999] = 38936,
	[13607] = 38799,
	[7418] = 38679,
	[25079] = 38889,
	[33993] = 38931,
	[34001] = 38937,
	[34009] = 38945,
	[44529] = 38967,
	[62948] = 45056,
	[28004] = 38927,
	[47901] = 39006,
	[34007] = 38943,
	[34008] = 38944,
	[21931] = 38876,
}


-- used to lookup the slot number of the enchant used by Scroll Master
-- index = itemID of enchant's scroll
-- value = slot number
Enchanting.slot = {
	[38897] = 3,
	[38913] = 4,
	[38929] = 4,
	[38945] = 7,
	[38961] = 2,
	[38977] = 5,
	[38993] = 5,
	[38770] = 5,
	[38786] = 2,
	[38802] = 6,
	[38818] = 4,
	[44455] = 7,
	[38850] = 6,
	[38866] = 4,
	[38882] = 3,
	[38898] = 3,
	[38914] = 5,
	[38930] = 4,
	[38946] = 9,
	[38962] = 4,
	[38978] = 5,
	[38994] = 9,
	[38771] = 3,
	[38787] = 7,
	[38803] = 3,
	[38819] = 2,
	[44456] = 5,
	[38851] = 6,
	[38867] = 4,
	[38883] = 9,
	[38899] = 3,
	[38915] = 5,
	[38931] = 6,
	[38947] = 9,
	[38963] = 9,
	[38979] = 6,
	[38995] = 9,
	[38772] = 1,
	[38788] = 1,
	[38804] = 4,
	[38820] = 7,
	[38836] = 6,
	[38852] = 3,
	[38868] = 9,
	[38884] = 9,
	[38900] = 3,
	[38932] = 6,
	[38948] = 9,
	[38964] = 6,
	[38980] = 3,
	[38996] = 3,
	[38773] = 4,
	[38789] = 5,
	[38805] = 7,
	[38821] = 9,
	[38837] = 2,
	[38853] = 3,
	[38869] = 1,
	[38885] = 6,
	[38901] = 3,
	[38917] = 9,
	[38933] = 6,
	[38949] = 7,
	[38965] = 9,
	[38981] = 1,
	[38997] = 3,
	[38774] = 3,
	[38790] = 5,
	[38806] = 5,
	[38822] = 1,
	[38838] = 9,
	[38854] = 3,
	[38870] = 9,
	[38886] = 6,
	[38902] = 3,
	[38918] = 9,
	[38934] = 6,
	[38950] = 5,
	[38966] = 2,
	[38982] = 5,
	[38998] = 9,
	[38679] = 3,
	[38775] = 5,
	[38791] = 7,
	[38807] = 2,
	[38823] = 6,
	[38839] = 7,
	[38855] = 3,
	[38871] = 9,
	[38887] = 6,
	[38903] = 3,
	[38919] = 1,
	[38935] = 6,
	[38951] = 6,
	[38967] = 6,
	[38983] = 7,
	[38999] = 4,
	[50816] = 6,
	[38776] = 4,
	[38792] = 7,
	[38808] = 4,
	[38824] = 4,
	[38840] = 9,
	[38856] = 6,
	[46026] = 9,
	[38888] = 6,
	[38904] = 7,
	[38920] = 9,
	[38936] = 6,
	[38968] = 3,
	[38984] = 3,
	[39000] = 5,
	[38777] = 3,
	[38793] = 3,
	[38809] = 3,
	[38825] = 5,
	[38841] = 4,
	[38857] = 6,
	[38873] = 9,
	[38889] = 6,
	[38905] = 7,
	[38921] = 9,
	[38937] = 3,
	[38953] = 6,
	[38969] = 5,
	[45628] = 2,
	[39001] = 5,
	[38778] = 3,
	[38794] = 9,
	[38810] = 2,
	[38826] = 5,
	[38842] = 3,
	[38858] = 5,
	[38874] = 1,
	[38890] = 6,
	[38906] = 7,
	[38922] = 1,
	[38938] = 3,
	[38954] = 7,
	[38970] = 6,
	[38986] = 2,
	[39002] = 4,
	[44815] = 3,
	[38779] = 9,
	[38795] = 5,
	[38811] = 3,
	[38827] = 6,
	[38843] = 7,
	[38859] = 5,
	[38875] = 1,
	[38891] = 5,
	[38907] = 7,
	[38923] = 9,
	[38939] = 5,
	[38955] = 4,
	[38971] = 3,
	[38987] = 3,
	[39003] = 5,
	[38780] = 9,
	[38796] = 1,
	[38812] = 3,
	[38828] = 7,
	[44465] = 4,
	[38860] = 7,
	[44497] = 9,
	[38892] = 5,
	[38908] = 2,
	[45056] = 8,
	[38940] = 5,
	[38956] = 5,
	[38972] = 9,
	[38988] = 9,
	[39004] = 5,
	[38781] = 1,
	[38797] = 3,
	[38813] = 9,
	[38829] = 3,
	[38845] = 1,
	[38861] = 7,
	[38877] = 9,
	[38893] = 5,
	[38909] = 2,
	[38925] = 9,
	[38941] = 5,
	[38957] = 9,
	[38973] = 5,
	[38989] = 4,
	[39005] = 4,
	[38766] = 4,
	[38782] = 4,
	[38798] = 4,
	[38814] = 9,
	[38830] = 2,
	[38846] = 3,
	[38862] = 2,
	[38878] = 9,
	[38894] = 5,
	[38910] = 2,
	[38926] = 9,
	[38942] = 5,
	[38958] = 9,
	[38974] = 2,
	[38990] = 6,
	[39006] = 2,
	[38767] = 4,
	[38783] = 3,
	[38799] = 4,
	[44947] = 3,
	[38831] = 6,
	[38847] = 4,
	[38863] = 2,
	[38879] = 9,
	[38895] = 5,
	[38911] = 4,
	[38927] = 9,
	[38943] = 2,
	[38959] = 5,
	[38975] = 4,
	[38991] = 9,
	[44493] = 9,
	[38876] = 9,
	[38844] = 2,
	[38768] = 3,
	[38784] = 5,
	[38800] = 6,
	[38816] = 7,
	[44453] = 9,
	[44469] = 2,
	[38864] = 2,
	[38880] = 9,
	[38896] = 1,
	[38912] = 4,
	[45060] = 8,
	[46098] = 9,
	[38960] = 6,
	[38976] = 2,
	[38992] = 1,
	[44470] = 3,
	[38834] = 6,
	[44463] = 1,
	[44466] = 9,
	[38815] = 5,
	[44457] = 5,
	[38848] = 9,
	[43987] = 9,
	[37603] = 2,
	[38832] = 3,
	[44458] = 6,
	[38872] = 9,
	[38928] = 4,
	[38944] = 2,
	[44449] = 2,
	[44467] = 9,
	[38835] = 5,
	[38769] = 4,
	[38785] = 2,
	[38801] = 6,
	[38817] = 3,
	[38833] = 4,
	[38849] = 3,
	[38865] = 4,
	[38881] = 3,
	[38924] = 9,
}

-- looks up the itemID of the lesser essence
-- index = itemID of greater essence
-- value = itemID of lesser essence
Enchanting.lesserEssence = {
	[11082] = 10998,
	[16203] = 16202,
	[10939] = 10938,
	[11175] = 11174,
	[22446] = 22447,
	[11135] = 11134,
	[34055] = 34056,
}

-- looks up the itemID of the greater essence
-- index = itemID of lesser essence
-- value = itemID of greater essence
Enchanting.greaterEssence = {
	[10998] = 11082,
	[11174] = 11175,
	[16202] = 16203,
	[10938] = 10939,
	[34056] = 34055,
	[22447] = 22446,
	[11134] = 11135,
}

-- looks up the name of a vellum
-- index = itemID of vellum
-- value = name of vellum
if GetLocale() == "deDE" then
	Enchanting.velName = "Rüstungspergament I"
elseif GetLocale() == "esES" then
	Enchanting.velName = "Vitela de arma"
elseif GetLocale() == "ruRU" then
	Enchanting.velName = "Материал для свитка брони"
elseif GetLocale() == "frFR" then
	Enchanting.velName = "Vélin d'arme"
elseif GetLocale() == "enUS" then
	Enchanting.velName = "Armor Vellum"
else
	Enchanting.velName = GetItemInfo(38682) or "Armor Vellum"
end